import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

class ReportLaborCostDetail(Model):
    _name="clinic.report.labor.cost.detail"
    _string="Report Labor Cost Detail"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'department_id': fields.Many2One("clinic.department","Department"),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        "type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),
        'date_from': _get_date_from,
        'date_to': _get_date_to,
        'type': 'doctor',
    }

    def get_report_data(self,ids,context={}):
        fmt='%Y-%m-%d'
        date_from=time.strftime(fmt)
        date_to=time.strftime(fmt)
        company_id=get_active_company()
        comp=get_model("company").browse(company_id)
        staff_type=None
        staff_id=None
        department_id=None
        defaults=context.get("defaults",{})
        if defaults:
            print('get from default ', defaults)
            date_from=defaults.get("date_from")
            date_to=defaults.get("date_to")
            department_id=defaults.get("department_id")
            staff_type=defaults.get("staff_type")
            staff_id=defaults.get("staff_id")
        dom=[]
        if ids:
            print("get from ids")
            obj=self.browse(ids)[0]
            date_from=obj.date_from
            date_to=obj.date_to
            staff_id=obj.staff_id.id
            staff_type=obj.type
            department_id=obj.department_id.id
        dom.append(['date','>=',date_from])
        dom.append(['date','<=',date_to])
        if staff_id:
            dom.append(['staff_id','=',staff_id])
        if staff_type:
            dom.append(['type','=',staff_type])
        if department_id:
            dom.append(['labor_cost_id.cycle_item_id.department_id','=',department_id])
        #prevent to load more data
        if not staff_id:
            return {}
        dates={}
        for line in get_model("clinic.labor.cost.line").search_browse(dom):
            lcost=line.labor_cost_id
            citem=lcost.cycle_item_id
            date=citem.date
            if not date:
                continue
            dpt=citem.department_id
            amt=line.amount or 0
            qty=len(citem.hd_cases) or 0 # qty of patient
            if not dates.get(date):
                dates[date]={
                    dpt.name: {
                        'amt': 0,
                        'qty': 0,
                        },
                    }
            if not dates[date].get(dpt.name):
                dates[date].update({
                        dpt.name: {
                            'amt': 0,
                            'qty': 0,
                        }})
            dates[date][dpt.name]['amt']+=amt
            dates[date][dpt.name]['qty']+=qty
        lines=[]
        dpts=get_model("clinic.department").search_read([],['name'])
        dpts=sorted(dpts, key=lambda b: b['name'])
        no=1
        dlines=sorted(dates.keys()) #sort by staff name
        for kdate in dlines:
            vals=dates[kdate]
            lvals={
                'no': no,
                'date': kdate,
            }
            total_qty, total_amt=0, 0
            lvals['sub_lines']=[]
            for dpt in dpts:
                amt,qty=0,0
                dname=dpt['name'] or ''
                dpt_vals=vals.get(dname)
                if dpt_vals:
                    amt=dpt_vals.get("amt",0)
                    qty=dpt_vals.get("qty",0)
                lvals['sub_lines'].append({'qty': qty, 'amt': amt})
                total_amt+=amt
                total_qty+=qty
            lvals['total_qty']=total_qty # total show as right hand side
            lvals['total_amt']=total_amt
            lines.append(lvals)
            no+=1
        # summary as footer
        dpt_lines=[{'qty': 0, 'amt': 0} for dpt in dpts]
        for line in lines:
            i=0
            for sub_line in line['sub_lines']:
                dpt_lines[i]['qty']+=sub_line['qty'] or 0
                dpt_lines[i]['amt']+=sub_line['amt'] or 0
                i+=1

        ## summary as footer (total)
        total_qty, total_amt=0, 0
        for tline in dpt_lines:
            total_qty+=tline['qty'] or 0
            total_amt+=tline['amt'] or 0
        dpt_lines.append({'amt': total_amt, 'qty': total_qty})
        
        for dpt in dpts:
            dpt.update({
                'qty_text': 'คนไข้',
                'amt_text': 'จำนวนเงิน',
            })
        comp_span=(len(dpts)*2)+1
        data={
            'date_from': date_from,
            'date_to': date_to,
            'dpts': dpts,
            'comp_name': comp.name or 0,
            'comp_span': comp_span, #qty, amt
            'lines': lines,
            'dpt_lines': dpt_lines,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

    def onchange_type(self,context={}):
        data=context['data']
        data['staff_id']=None
        return data

ReportLaborCostDetail.register()
